@@ -28,6 +28,7 @@
 
 #include <Foundation/NSBundle.h>
 #include <Foundation/NSDictionary.h>
+#include <Foundation/NSEnumerator.h>
 #include <Foundation/NSString.h>
 #include <Foundation/NSProcessInfo.h>
 
@@ -37,6 +38,7 @@
 #include "AppKit/NSFont.h"
 #include "AppKit/NSImage.h"
 #include "AppKit/NSImageView.h"
+#include "AppKit/NSPasteboard.h"
 #include "AppKit/NSTextField.h"
 #include "GNUstepGUI/GSInfoPanel.h"
 #include "GNUstepGUI/GSTheme.h"
@@ -212,13 +214,17 @@ new_label (NSString *value)
   if (nil_or_not_of_class (name, [NSString class]))
     {
       name = value_from_info_plist_for_key (@"ApplicationName");
-      
       if (nil_or_not_of_class (name, [NSString class]))
 	{
 	  name = value_from_info_plist_for_key (@"NSHumanReadableShortName");
-
 	  if (nil_or_not_of_class (name, [NSString class]))
-	    name = [[NSProcessInfo processInfo] processName]; 
+	    {
+	      name = value_from_info_plist_for_key (@"CFBundleName");
+	      if (nil_or_not_of_class (name, [NSString class]))
+		{
+		  name = [[NSProcessInfo processInfo] processName]; 
+		}
+	    }
 	}
     }
   /* Application Description */
@@ -263,7 +269,14 @@ new_label (NSString *value)
 	      release = value_from_info_plist_for_key (@"NSAppVersion");
 	      
 	      if (nil_or_not_of_class (release, [NSString class]))
-		release = @"Unknown";
+		{
+		  release = value_from_info_plist_for_key (@"CFBundleVersion");
+	      
+		  if (nil_or_not_of_class (release, [NSString class]))
+		    {
+		      release = @"Unknown";
+		    }
+		}
 	    }
 	}
     }
@@ -300,17 +313,19 @@ new_label (NSString *value)
   if (nil_or_not_of_class (authors, [NSArray class]))
     {
       authors = value_from_info_plist_for_key (@"Authors");
-
-      if (nil_or_not_of_class (authors, [NSArray class]))
-	authors = [NSArray arrayWithObject: @"Unknown"];
+      
+      // if (nil_or_not_of_class (authors, [NSArray class]))
+      //   authors = [NSArray arrayWithObject: @"Unknown"];
     }
   /* URL */
   if (dictionary)
-    url = [dictionary objectForKey: @"URL"];
+      url = [dictionary objectForKey: @"URL"];
 
-  if ([url isKindOfClass: [NSString class]] == NO)
-    url = nil;
-  /* NB: URL can be nil */
+  if (nil_or_not_of_class (url, [NSString class]))
+    {
+      url = value_from_info_plist_for_key (@"URL");
+    }
+  // URL can be nil
 
   /* Copyright */
   if (dictionary)
@@ -374,7 +389,11 @@ new_label (NSString *value)
   [versionLabel setFont: smallFont];
   [versionLabel sizeToFit];
 
-  if ([authors count] == 1)
+  if ([authors count] == 0)
+    {
+      authorTitleLabel = new_label (@"");
+    }
+  else if ([authors count] == 1)
     {
       authorTitleLabel = new_label (@"Author: ");
     }
@@ -645,4 +664,50 @@ new_label (NSString *value)
   return self;
 }
 
+- (void) copy: (id)sender
+{  
+  NSArray *types = [NSArray arrayWithObject: NSStringPboardType];
+  NSPasteboard *pboard = [NSPasteboard generalPasteboard];
+  NSMutableString *text = [[NSMutableString alloc] init];
+  NSView *cv = [self contentView];
+  NSEnumerator *enumerator = [[cv subviews] objectEnumerator];
+  NSView *subview;
+ 
+  // Loop over all the text subviews and collect the information
+  while ((subview = [enumerator nextObject]) != nil)
+    {
+      if ([subview isKindOfClass: [NSTextField class]])
+        {
+          [text appendString: [(NSTextField*)subview stringValue]];
+          [text appendString: @"\n"];
+        }
+    }
+  
+  [pboard declareTypes: types owner: self];
+  [pboard setString: text
+          forType: NSStringPboardType];
+  RELEASE(text);
+}
+
+- (void) keyDown: (NSEvent*)theEvent
+{
+  NSString *characters = [theEvent characters];
+  unichar character = 0;
+
+  if ([characters length] > 0)
+    {
+      character = [characters characterAtIndex: 0];
+    }
+
+  // FIXME: Hard coded
+  if (character == 'c' && 
+      [theEvent modifierFlags] & NSCommandKeyMask)
+    {
+      [self copy: nil];
+      return;
+    }
+
+  [super keyDown: theEvent];
+}
+
 @end
