@@ -47,9 +47,16 @@
 /* And another so that boolean is not redefined in jmorecfg.h. */
 #ifndef HAVE_BOOLEAN
 #define HAVE_BOOLEAN
+/* This MUST match the jpeg definition of boolean */
+typedef int jpeg_boolean;
+#define boolean jpeg_boolean
 #endif
 #endif
 #include <jpeglib.h>
+#if defined(__CYGWIN__)
+/* Cygwin uses a patched jpeg */
+#define GSTEP_PROGRESSIVE_CODEC
+#endif
 
 #include <setjmp.h>
 
@@ -465,7 +472,11 @@ static void gs_jpeg_memory_dest_destroy (j_compress_ptr cinfo)
         }
     }
 
+#ifdef GSTEP_PROGRESSIVE_CODEC
+  isProgressive = (cinfo.process == JPROC_PROGRESSIVE);
+#else
   isProgressive = cinfo.progressive_mode;
+#endif
 
   /* done */
   jpeg_finish_decompress(&cinfo);
@@ -599,7 +610,12 @@ static void gs_jpeg_memory_dest_destroy (j_compress_ptr cinfo)
   progressiveNumber = [properties objectForKey: NSImageProgressive];
   if (progressiveNumber != nil)
     {
+#ifdef GSTEP_PROGRESSIVE_CODEC
+      if ([progressiveNumber boolValue])
+        cinfo.process = JPROC_PROGRESSIVE;
+#else
       cinfo.progressive_mode = [progressiveNumber boolValue];
+#endif
     }
 
 
