@@ -28,6 +28,7 @@
 
 #include "config.h"
 #include <Foundation/NSNotification.h>
+#include <Foundation/NSException.h>
 
 #include "AppKit/NSClipView.h"
 #include "AppKit/NSCursor.h"
@@ -38,10 +39,16 @@
 #include "AppKit/NSWindow.h"
 #include "AppKit/PSOperators.h"
 
+#include <GNUstepGUI/GSNibLoading.h>
+
 #include <math.h>
 
 DEFINE_RINT_IF_MISSING
 
+@interface NSClipView (Private)
+- (void) _scrollToPoint: (NSPoint)aPoint;
+@end
+
 /*
  * Return the biggest integral (in device space) rect contained in rect. 
  * Conversion to/from device space is done using view.
@@ -538,7 +545,7 @@ static inline NSRect integralRect (NSRect rect, NSView *view)
  */
 - (void) viewFrameChanged: (NSNotification*)aNotification
 {
-  [self scrollToPoint: _bounds.origin];
+  [self _scrollToPoint: _bounds.origin];
 
   /* If document frame does not completely cover _bounds */
   if (NSContainsRect([_documentView frame], _bounds) == NO)
@@ -785,6 +792,9 @@ static inline NSRect integralRect (NSRect rect, NSView *view)
       if ([[self subviews] count] > 0)
         {
 	  id document = [aDecoder decodeObjectForKey: @"NSDocView"];
+
+	  NSAssert([document class] != [NSCustomView class],
+		   NSInvalidArgumentException);
 	  NSRect rect = [document frame];
 	  rect.origin = NSZeroPoint;
 	  [document setFrame: rect];
@@ -817,3 +827,26 @@ static inline NSRect integralRect (NSRect rect, NSView *view)
   return self;
 }
 @end
+
+@implementation NSClipView (Private)
+
+- (void) _scrollToPoint: (NSPoint)aPoint
+{ 
+  NSRect proposedBounds; 
+  NSRect proposedVisibleRect; 
+  NSRect newVisibleRect; 
+  NSRect newBounds; 
+  
+  // give documentView a chance to adjust its visible rectangle 
+  proposedBounds = _bounds; 
+  proposedBounds.origin = aPoint; 
+  proposedVisibleRect = [self convertRect: proposedBounds 
+                              toView: _documentView]; 
+  newVisibleRect = [_documentView adjustScroll: proposedVisibleRect]; 
+  newBounds = [self convertRect: newVisibleRect fromView: _documentView]; 
+  
+  [self scrollToPoint: newBounds.origin]; 
+}
+
+@end
+
