@@ -10,19 +10,20 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
 */
 
 #include <Foundation/NSNotification.h>
@@ -40,6 +41,7 @@
 #include "AppKit/NSButtonCell.h"
 #include "AppKit/NSComboBox.h"
 #include "AppKit/NSComboBoxCell.h"
+#include "AppKit/NSEvent.h"
 #include "AppKit/NSGraphicsContext.h"
 #include "AppKit/NSImage.h"
 #include "AppKit/NSMatrix.h"
@@ -1447,6 +1449,16 @@ static GSComboWindow *gsWindow = nil;
   _completes = completes;
 }
 
+- (BOOL) isButtonBordered
+{
+  return [_buttonCell isBordered];
+}
+
+- (void) setButtonBordered:(BOOL)flag
+{
+  [_buttonCell setBordered: flag];
+}
+
 #define ComboBoxHeight 21 // FIX ME: All this stuff shouldn't be hardcoded
 #define ButtonWidth 17
 #define ButtonHeight 17
