@@ -634,6 +634,11 @@ static NSNotificationCenter *nc;
    */
   if ([self currentEditor] != nil)
     {
+      /* Make sure the cell's control view is always set up.
+       * FIXME Need to draw the cell's border which is not covered by
+       *       the field editor if the cell is bordered.
+       */
+      [_cell setControlView: self];
       return;
     }
 
@@ -671,7 +676,7 @@ static NSNotificationCenter *nc;
 {
   if (_cell == aCell)
     {
-      [_cell setState: 1];
+      [_cell setState: NSOnState];
       [self setNeedsDisplay: YES];
     }
 }
@@ -966,7 +971,11 @@ static NSNotificationCenter *nc;
 
 - (id) initWithCoder: (NSCoder*)aDecoder
 {
-  [super initWithCoder: aDecoder];
+  self = [super initWithCoder: aDecoder];
+  if (self == nil)
+    {
+      return nil;
+    }
 
   if ([aDecoder allowsKeyedCoding])
     {
@@ -976,6 +985,15 @@ static NSNotificationCenter *nc;
         {
           [self setCell: cell];
         }
+      else
+        {
+          // This is needed for subclasses without cells, like NSColorWeel
+          // as we store some properties only on the cell.
+          NSCell *cell = [[[self class] cellClass] new];
+
+          [self setCell: cell];
+          RELEASE(cell);
+        }
       if ([aDecoder containsValueForKey: @"NSEnabled"])
         {
           [self setEnabled: [aDecoder decodeBoolForKey: @"NSEnabled"]];
