@@ -32,9 +32,11 @@
 #include <Foundation/NSFileManager.h>
 #include <Foundation/NSNotification.h>
 #include <Foundation/NSPathUtilities.h>
+#include <Foundation/NSProcessInfo.h>
 #include <Foundation/NSString.h>
 #include <Foundation/NSURL.h>
 #include <Foundation/NSUserDefaults.h>
+#include <Foundation/NSTimer.h>
 
 #include "AppKit/NSDocumentController.h"
 #include "AppKit/NSOpenPanel.h"
@@ -54,8 +56,12 @@ static NSString *NSDOSExtensionsKey     = @"NSDOSExtensions";
 //static NSString *NSMIMETypesKey         = @"NSMIMETypes";
 static NSString *NSDocumentClassKey     = @"NSDocumentClass";
 
+static NSString *CFBundleDocumentTypes  = @"CFBundleDocumentTypes";
+static NSString *CFBundleTypeExtensions = @"CFBundleTypeExtensions";
+static NSString *CFBundleTypeName       = @"CFBundleTypeName";
+static NSString *CFBundleTypeRole       = @"CFBundleTypeRole";
+
 static NSString *NSRecentDocuments      = @"NSRecentDocuments";
-static NSString *NSDefaultOpenDirectory = @"NSDefaultOpenDirectory";
 
 static NSDocumentController *sharedController = nil;
 
@@ -73,6 +79,10 @@ static NSDictionary *TypeInfoForName (NSArray *types, NSString *typeName)
 	{
 	  return dict;
 	}
+      else if ([[dict objectForKey: CFBundleTypeName] isEqualToString: typeName])
+	{
+	  return dict;
+	}
     }  
 
   return nil;
@@ -89,6 +99,10 @@ static NSDictionary *TypeInfoForHumanReadableName (NSArray *types, NSString *typ
 	{
 	  return dict;
 	}
+      else if ([[dict objectForKey: CFBundleTypeName] isEqualToString: typeName])
+	{
+	  return dict;
+	}
     }  
 
   return nil;
@@ -156,8 +170,9 @@ static NSDictionary *TypeInfoForHumanReadableName (NSArray *types, NSString *typ
 */
 + (BOOL) isDocumentBasedApplication
 {
-  return ([[[NSBundle mainBundle] infoDictionary] objectForKey: NSTypesKey])
-	? YES : NO;
+  return ([[[NSBundle mainBundle] infoDictionary] objectForKey: NSTypesKey] ||
+	  [[[NSBundle mainBundle] infoDictionary] objectForKey: CFBundleDocumentTypes]) 
+    ? YES : NO;
 }
 
 /** </init>Initializes the document controller class. The first
@@ -169,6 +184,12 @@ static NSDictionary *TypeInfoForHumanReadableName (NSArray *types, NSString *typ
   NSDictionary *customDict = [[NSBundle mainBundle] infoDictionary];
 	
   ASSIGN (_types, [customDict objectForKey: NSTypesKey]);
+
+  if(_types == nil)
+    {
+      ASSIGN(_types, [customDict objectForKey: CFBundleDocumentTypes]);
+    }
+
   _documents = [[NSMutableArray alloc] init];
   
   /* Get list of recent documents */
@@ -259,7 +280,24 @@ static NSDictionary *TypeInfoForHumanReadableName (NSArray *types, NSString *typ
 
 - (void) setAutosavingDelay: (NSTimeInterval)autosavingDelay
 {
+  static NSTimer *autosavingTimer;
+
+  if (autosavingTimer)
+    {
+      [autosavingTimer invalidate];
+      DESTROY (autosavingTimer);
+    }
   _autosavingDelay = autosavingDelay;
+  if (autosavingDelay > 0)
+    {
+      autosavingTimer =
+	[NSTimer scheduledTimerWithTimeInterval: autosavingDelay
+		 target: self
+		 selector: @selector(_autosaveDocuments:)
+		 userInfo: nil
+		 repeats: YES];
+      RETAIN (autosavingTimer);
+    }
 }
 
 - (id) makeUntitledDocumentOfType: (NSString *)type
@@ -344,12 +382,19 @@ static NSDictionary *TypeInfoForHumanReadableName (NSArray *types, NSString *typ
 
 - (NSString*) defaultType
 {
+  NSString *defaultName = nil;
   if ([_types count] == 0) 
     {
       return nil; // raise exception?
     }
   
-  return [(NSDictionary*)[_types objectAtIndex: 0] objectForKey: NSNameKey];
+  defaultName = [(NSDictionary*)[_types objectAtIndex: 0] objectForKey: NSNameKey];
+  if(defaultName == nil)
+    {
+      defaultName = [(NSDictionary*)[_types objectAtIndex: 0] objectForKey: CFBundleTypeName];
+    }
+
+  return defaultName;
 }
 
 - (void) addDocument: (NSDocument *)document
@@ -546,7 +591,32 @@ static NSDictionary *TypeInfoForHumanReadableName (NSArray *types, NSString *typ
             withContentsOfURL: (NSURL *)contents
                         error: (NSError **)err
 {
-  // FIXME
+  if ([contents isFileURL])
+    {
+      NSString *type =
+	  [self typeFromFileExtension: [[contents path] pathExtension]];
+      id document =
+	  [self makeDocumentForURL: url
+		withContentsOfURL: contents
+		ofType: type
+		error: err];
+      if (document)
+        {
+	  [self addDocument:document];
+	  if ([self shouldCreateUI])
+	    {
+	      [document makeWindowControllers];
+	      [document showWindows];
+	    }  
+	  return YES;
+	}
+    }
+  else
+    {
+      // FIXME: set error
+      *err = nil;
+    }
+
   return NO;
 }
 
@@ -577,6 +647,7 @@ static NSDictionary *TypeInfoForHumanReadableName (NSArray *types, NSString *typ
       NSDictionary *typeInfo = [_types objectAtIndex: i];
       [array addObjectsFromArray: [typeInfo objectForKey: NSUnixExtensionsKey]];
       [array addObjectsFromArray: [typeInfo objectForKey: NSDOSExtensionsKey]];
+      [array addObjectsFromArray: [typeInfo objectForKey: CFBundleTypeExtensions]];
     }
   
   return array;
@@ -700,10 +771,6 @@ static NSDictionary *TypeInfoForHumanReadableName (NSArray *types, NSString *typ
   NSString *cancelString = (cancellable)? ((NSString *)_(@"Cancel")) : ((NSString *)nil);
   int      result;
   
-  /* Probably as good a place as any to do this */
-  [[NSUserDefaults standardUserDefaults] 
-    setObject: [self currentDirectory] forKey: NSDefaultOpenDirectory];
-
   if (![self hasEditedDocuments]) 
     {
       return YES;
@@ -822,13 +889,19 @@ static NSDictionary *TypeInfoForHumanReadableName (NSArray *types, NSString *typ
 {
   NSFileManager *manager = [NSFileManager defaultManager];
   NSDocument *document = [self currentDocument];
-  NSString *directory;
+  NSString *directory = nil;
   BOOL isDir = NO;
 
   if (document)
-    directory = [[document fileName] stringByDeletingLastPathComponent];
-  else
-    directory = [[NSOpenPanel openPanel] directory];
+    {
+      directory = [[document fileName] stringByDeletingLastPathComponent];
+    }
+  if (directory == nil || [directory isEqual: @""]
+      || [manager fileExistsAtPath: directory  isDirectory: &isDir] == NO
+      || isDir == NO)
+    {
+      directory = [[NSOpenPanel openPanel] directory];
+    }
   if (directory == nil || [directory isEqual: @""]
       || [manager fileExistsAtPath: directory  isDirectory: &isDir] == NO
       || isDir == NO)
@@ -943,9 +1016,16 @@ static NSDictionary *TypeInfoForHumanReadableName (NSArray *types, NSString *typ
       if ([[typeInfo objectForKey:NSUnixExtensionsKey] 
 	    containsObject: fileExtension] ||
 	  [[typeInfo objectForKey:NSDOSExtensionsKey]  
+	    containsObject: fileExtension] ||
+	  [[typeInfo objectForKey:CFBundleTypeExtensions]
 	    containsObject: fileExtension])
 	{
-	  return [typeInfo objectForKey: NSNameKey];
+	  NSString *type = [typeInfo objectForKey: NSNameKey];
+	  if(type == nil)
+	    {
+	      type = [typeInfo objectForKey: CFBundleTypeName];
+	    }
+	  return type;
 	}
     }
 	
@@ -966,10 +1046,14 @@ static NSDictionary *TypeInfoForHumanReadableName (NSArray *types, NSString *typ
   NSDictionary *typeInfo = TYPE_INFO(type);
   NSArray *unixExtensions = [typeInfo objectForKey: NSUnixExtensionsKey];
   NSArray *dosExtensions  = [typeInfo objectForKey: NSDOSExtensionsKey];
-  
+  NSArray *cfFileExtensions = [typeInfo objectForKey: CFBundleTypeExtensions];
+
+  if (!dosExtensions && !unixExtensions) return cfFileExtensions;
   if (!dosExtensions)  return unixExtensions;
   if (!unixExtensions) return dosExtensions;
-  return [unixExtensions arrayByAddingObjectsFromArray: dosExtensions];
+
+  return [[unixExtensions arrayByAddingObjectsFromArray: dosExtensions] 
+	   arrayByAddingObjectsFromArray: cfFileExtensions];
 }
 
 - (Class) documentClassForType: (NSString *)type
@@ -989,8 +1073,17 @@ static NSDictionary *TypeInfoForHumanReadableName (NSArray *types, NSString *typ
   for (i = 0; i < count; i++)
     {
       NSDictionary *typeInfo = [_types objectAtIndex: i];
-      
-      [classNames addObject: [typeInfo objectForKey: NSDocumentClassKey]];
+      NSString *className = [typeInfo objectForKey: NSDocumentClassKey];
+
+      if(className != nil)
+	{
+	  [classNames addObject: className];
+	}
+      else
+	{
+	  NSLog(@"WARN: The entry does not have an NSDocumentClass name defined: %@",
+		typeInfo);
+	}
     }
  
   return AUTORELEASE(classNames);
@@ -1072,12 +1165,23 @@ static NSString *NSViewerRole = @"Viewer";
       NSString     *className = [typeInfo objectForKey: NSDocumentClassKey];
       NSString     *role      = [typeInfo objectForKey: NSRoleKey];
       
+      // if the standard one isn't filled... check the CF key.
+      if(role == nil)
+	{
+	  role = [typeInfo objectForKey: CFBundleTypeRole];
+	}
+
       if ([docClassName isEqualToString: className] 
 	  && (role == nil 
 	      || [role isEqual: NSEditorRole] 
 	      || [role isEqual: NSViewerRole]))
 	{
-	  [types addObject: [typeInfo objectForKey: NSNameKey]];
+	  NSString *name = [typeInfo objectForKey: NSNameKey];
+	  if(name == nil)
+	    {
+	      name = [typeInfo objectForKey: CFBundleTypeName];
+	    }
+	  [types addObject: name];
 	}
     }
   
@@ -1099,7 +1203,12 @@ static NSString *NSViewerRole = @"Viewer";
       if ([docClassName isEqualToString: className] &&
 	  (role == nil || [role isEqual: NSEditorRole]))
 	{
-	  [types addObject: [typeInfo objectForKey: NSNameKey]];
+	  NSString *name = [typeInfo objectForKey: NSNameKey];
+	  if(name == nil)
+	    {
+	      name = [typeInfo objectForKey: CFBundleTypeName];
+	    }
+	  [types addObject: name];
 	}
     }
   
@@ -1135,5 +1244,152 @@ static NSString *NSViewerRole = @"Viewer";
   return [self _displayNamesForTypes: 
 		 [self _editorTypesForClass: documentClass]];
 }
+
+static NSMapTable *autosavedDocuments;
+static NSString *processName;
+
+- (NSString *) _autosaveDirectory: (BOOL)create
+{
+  NSArray *paths =
+      NSSearchPathForDirectoriesInDomains (NSLibraryDirectory,
+                                           NSUserDomainMask,
+                                           YES);
+  NSString *path = [paths objectAtIndex:0];
+  path = [path stringByAppendingPathComponent: @"Autosave"];
+
+  if (create)
+    {
+      BOOL isDir;
+      NSFileManager *fm = [NSFileManager defaultManager];
+
+      if ([fm fileExistsAtPath: path isDirectory: &isDir] == NO)
+        {
+          if (![fm createDirectoryAtPath: path attributes: nil])
+            return nil;
+        }
+      else if (isDir == NO)
+        {
+          if (![fm removeFileAtPath: path handler: nil] ||
+              ![fm createDirectoryAtPath: path attributes: nil])
+            return nil;
+        }
+    }
+
+  return path;
+}
+
+- (void) _autosaveDocuments: (NSTimer *)timer
+{
+  id document;
+  int i, n = [_documents count];
+
+  for (i = 0; i < n; i++)
+    {
+      document = [_documents objectAtIndex: i];
+      if ([document autosavingFileType] && [document hasUnautosavedChanges])
+        {
+	  [document autosaveDocumentWithDelegate: nil
+		    didAutosaveSelector: NULL
+		    contextInfo: NULL];
+	}
+    }
+}
+
+- (BOOL) _reopenAutosavedDocuments
+{
+  BOOL didOpen = NO;
+
+  if (!autosavedDocuments)
+    {
+      NSArray *autosaved;
+      NSString *path;
+
+      autosavedDocuments =
+          NSCreateMapTable (NSObjectMapKeyCallBacks,
+                            NSObjectMapValueCallBacks,
+                            1);
+      if (!processName)
+        processName = [[[NSProcessInfo processInfo] processName] copy];
+
+      path = [self _autosaveDirectory: NO];
+      path = [path stringByAppendingPathComponent: processName];
+      path = [path stringByAppendingPathExtension: @"plist"];
+      autosaved = [NSArray arrayWithContentsOfFile: path];
+      if (autosaved)
+        {
+          int i, n = [autosaved count];
+          NSFileManager *fm = [NSFileManager defaultManager];
+
+          for (i = 0; i < n; i++)
+            {
+              NSDictionary *dict = [autosaved objectAtIndex: i];
+              NSString *location = [dict objectForKey: @"Location"];
+              NSString *autosavedLoc = [dict objectForKey: @"AutosavedLocation"];
+              NSURL *url = location ? [NSURL URLWithString: location] : nil;
+              NSURL *autosavedURL =
+                  autosavedLoc ? [NSURL URLWithString: autosavedLoc] : nil;
+
+              if (autosavedURL && [fm fileExistsAtPath: [autosavedURL path]])
+                {
+                  NSError *err;
+                  if ([self reopenDocumentForURL: url
+                            withContentsOfURL: autosavedURL
+                            error: &err])
+                    didOpen = YES;
+                }
+            }
+        }
+    }
+
+  return didOpen;
+}
+
+- (void) _recordAutosavedDocument: (NSDocument *)document
+{
+  BOOL changed = NO;
+  NSURL *url = [document autosavedContentsFileURL];
+
+  if (!autosavedDocuments)
+    autosavedDocuments =
+        NSCreateMapTable (NSObjectMapKeyCallBacks,
+                          NSObjectMapValueCallBacks,
+                          1);
+  if (!processName)
+    processName = [[[NSProcessInfo processInfo] processName] copy];
+
+  if (url)
+    {
+      NSMutableDictionary *dict = [[NSMutableDictionary alloc] init];
+      if ([document fileURL])
+        [dict setObject: [document fileURL] forKey: @"Location"];
+      [dict setObject: url forKey: @"AutosavedLocation"];
+      NSMapInsert (autosavedDocuments, document, dict);
+      [dict release];
+      changed = YES;
+    }
+  else if (NSMapGet (autosavedDocuments, document))
+    {
+      NSMapRemove (autosavedDocuments, document);
+      changed = YES;
+    }
+
+  if (changed)
+    {
+      NSString *path = [self _autosaveDirectory: YES];
+      NSArray *autosaved = NSAllMapTableValues (autosavedDocuments);
+      NSFileManager *fm = [NSFileManager defaultManager];
+
+      path = [path stringByAppendingPathComponent: processName];
+      path = [path stringByAppendingPathExtension: @"plist"];
+      if ([autosaved count] == 0)
+        {
+          [fm removeFileAtPath: path handler: nil];
+        }
+      else
+        {
+          [autosaved writeToFile: path atomically:YES];
+        }
+    }
+}
 @end
 
