@@ -145,8 +145,9 @@ static BOOL menuBarVisible = YES;
 
 - (NSString*) _locationKey
 {
-  if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil)
-    == NSMacintoshInterfaceStyle)
+  NSInterfaceStyle style = NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil);
+  if (style == NSMacintoshInterfaceStyle || 
+      style == NSWindows95InterfaceStyle)
     {
       return nil;
     }
@@ -290,8 +291,8 @@ static BOOL menuBarVisible = YES;
 	  [[appMenu menuRepresentation] update];
 
 	  RELEASE(itemsToMove);
-	}
-      else
+	}      
+      else 
 	{
 	  [appItem setImage: nil];
 	  if (appMenu != nil)
@@ -315,7 +316,7 @@ static BOOL menuBarVisible = YES;
 		}
 	      [self removeItem: appItem];
 	    }
-	}
+	}  
     }
 
   for (i = 0; i < [_items count]; i++)
@@ -815,12 +816,13 @@ static BOOL menuBarVisible = YES;
       NSMenuItem *menuItem = [_items objectAtIndex: i];
 
       if (actionSelector == 0 || sel_eq([menuItem action], actionSelector))
-	{
-	  if ([[menuItem target] isEqual: anObject])
-	    {
-	      return i;
-	    }
-	}
+        {
+          // There are different possibilities to implement the check here
+            if ([menuItem target] == anObject)
+            {
+              return i;
+            }
+        }
     }
 
   return -1;
@@ -833,10 +835,10 @@ static BOOL menuBarVisible = YES;
   for (i = 0; i < count; i++)
     {
       if ([[[_items objectAtIndex: i] representedObject]
-	isEqual: anObject])
-	{
-	  return i;
-	}
+              isEqual: anObject])
+        {
+          return i;
+        }
     }
 
   return -1;
@@ -851,10 +853,10 @@ static BOOL menuBarVisible = YES;
       id item = [_items objectAtIndex: i];
 
       if ([item hasSubmenu] && 
-	  [[item submenu] isEqual: anObject])
-	{
-	  return i;
-	}
+          [[item submenu] isEqual: anObject])
+        {
+          return i;
+        }
     }
   
   return -1;
@@ -1382,7 +1384,7 @@ static BOOL menuBarVisible = YES;
       [encoder encodeObject: _items forKey: @"NSMenuItems"];
       
       // if there is no supermenu, make it the main menu.
-      if ([self supermenu] == nil)
+      if ([self supermenu] == nil && ![self _ownedByPopUp])
 	{
 	  [encoder encodeObject: @"_NSMainMenu" forKey: @"NSName"];
 	}
@@ -1486,8 +1488,9 @@ static BOOL menuBarVisible = YES;
 
 - (void) _showTornOffMenuIfAny: (NSNotification*)notification
 {
-  if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil)
-    == NSMacintoshInterfaceStyle)
+  NSInterfaceStyle style = NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil);
+  if (style == NSMacintoshInterfaceStyle || 
+      style == NSWindows95InterfaceStyle)
     {
       return;
     }
@@ -1727,7 +1730,8 @@ static BOOL menuBarVisible = YES;
 	  NSMenuView	*newRep;
 
 	  newRep = [[NSMenuView alloc] initWithFrame: NSZeroRect];
-	  if (newStyle == NSMacintoshInterfaceStyle)
+	  if (newStyle == NSMacintoshInterfaceStyle ||
+	      newStyle == NSWindows95InterfaceStyle)
 	    {
 	      [newRep setHorizontal: YES];
 	    }
