@@ -284,6 +284,22 @@ this to return nil to indicate that we have no context menu.
   [self synchronizeTitleAndSelectedItem];
 }
 
+- (BOOL) selectItemWithTag: (NSInteger)tag
+{
+   int index = [self indexOfItemWithTag: tag];
+
+   if (index >= 0)
+     {
+       [self selectItemAtIndex: index];
+       return YES;
+     }
+   else
+     {
+       return NO;
+     }
+}
+
+
 /** <p>Returns the number of items in the item list</p>
     <p>See Also: [NSPopUpButtonCell-numberOfItems]</p>
  */
@@ -452,6 +468,14 @@ this to return nil to indicate that we have no context menu.
 	case NSNewlineCharacter:
 	case NSEnterCharacter: 
 	case NSCarriageReturnCharacter:
+	  /* Handle Enter and Return keys only when the menu is visible.
+	     The button's action to pop up the menu is initiated only by
+	     the Space key similar to other buttons. */
+	  {
+	    NSMenuView *menuView = [[_cell menu] menuRepresentation];
+	    if ([[menuView window] isVisible] == NO)
+	      break;
+	  }
 	case ' ':
 	  {
 	    int selectedIndex;
