@@ -108,7 +108,9 @@ setPath(NSBrowser *browser, NSString *path)
 - (id) _initWithoutGModel;
 - (void) _getOriginalSize;
 - (void) _setDefaultDirectory;
+- (void) _updateDefaultDirectory;
 - (void) _resetDefaults;
+- (void) _reloadBrowser;
 // Methods invoked by buttons
 - (void) _setHomeDirectory;
 - (void) _mountMedia;
@@ -116,7 +118,7 @@ setPath(NSBrowser *browser, NSString *path)
 - (void) _selectTextInColumn: (int)column;
 - (void) _selectCellName: (NSString *)title;
 - (void) _setupForDirectory: (NSString *)path file: (NSString *)name;
-- (BOOL) _shouldShowExtension: (NSString *)extension isDir: (BOOL *)isDir;
+- (BOOL) _shouldShowExtension: (NSString *)extension;
 - (void) _windowResized: (NSNotification*)n;
 - (NSComparisonResult) _compareFilename: (NSString *)n1 with: (NSString *)n2;
 @end /* NSSavePanel (PrivateMethods) */
@@ -149,9 +151,7 @@ setPath(NSBrowser *browser, NSString *path)
       NSString	*file = [[names lastObject] stringByStandardizingPath];
       BOOL isDir;
 
-      if (file && 
-	  [_fm fileExistsAtPath: file isDirectory: &isDir] && 
-	  isDir)
+      if (file && [_fm fileExistsAtPath: file isDirectory: &isDir] && isDir)
         {
 	  [self setDirectory: file];
 	}
@@ -249,7 +249,6 @@ setPath(NSBrowser *browser, NSString *path)
 
   r = NSMakeRect (43, 6, 27, 27);
   button = [[NSButton alloc] initWithFrame: r];
-  [button setRefusesFirstResponder: YES];
   [button setBordered: YES];
   image = [NSImage imageNamed: @"common_Home"];
   [button setImage: image];
@@ -265,7 +264,6 @@ setPath(NSBrowser *browser, NSString *path)
   
   r = NSMakeRect (78, 6, 27, 27);
   button = [[NSButton alloc] initWithFrame: r];
-  [button setRefusesFirstResponder: YES];
   [button setBordered: YES];
   image = [NSImage imageNamed: @"common_Mount"]; 
   [button setImage: image]; 
@@ -281,7 +279,6 @@ setPath(NSBrowser *browser, NSString *path)
 
   r = NSMakeRect (112, 6, 27, 27);
   button = [[NSButton alloc] initWithFrame: r];
-  [button setRefusesFirstResponder: YES];
   [button setBordered: YES];
   image = [NSImage imageNamed: @"common_Unmount"]; 
   [button setImage: image];
@@ -297,7 +294,6 @@ setPath(NSBrowser *browser, NSString *path)
   
   r = NSMakeRect (148, 6, 71, 27);
   button = [[NSButton alloc] initWithFrame: r]; 
-  [button setRefusesFirstResponder: YES];
   [button setBordered: YES];
   [button setTitle:  @"Cancel"];
   [button setImagePosition: NSNoImage]; 
@@ -305,6 +301,8 @@ setPath(NSBrowser *browser, NSString *path)
   [button setAction: @selector(cancel:)];
   [button setAutoresizingMask: NSViewMinXMargin];
   [button setTag: NSFileHandlingPanelCancelButton];
+  [button setKeyEquivalent: @"\e"];
+  [button setKeyEquivalentModifierMask: 0];
   [_bottomView addSubview: button];
   [lastKeyView setNextKeyView: button];
   lastKeyView = button;
@@ -312,7 +310,6 @@ setPath(NSBrowser *browser, NSString *path)
   
   r = NSMakeRect (228, 6, 71, 27);
   _okButton = [[NSButton alloc] initWithFrame: r]; 
-  [button setRefusesFirstResponder: YES];
   [_okButton setBordered: YES];
   [_okButton setTitle:  @"OK"];
   [_okButton setImagePosition: NSImageRight]; 
@@ -389,20 +386,24 @@ setPath(NSBrowser *browser, NSString *path)
 {
   NSString *path;
 
-  if (_directory == nil)
+  path = [[NSUserDefaults standardUserDefaults] 
+	     objectForKey: @"NSDefaultOpenDirectory"];
+  if (path == nil)
     {
-      path = [[NSUserDefaults standardUserDefaults] 
-		 objectForKey: @"NSDefaultOpenDirectory"];
-      if (path == nil)
-        {
-	  // FIXME: Should we use this or the home directory?
-	  ASSIGN(_directory, [_fm currentDirectoryPath]);
-	}
-      else
-        {
-	  ASSIGN(_directory, path);
-	}
+      // FIXME: Should we use this or the home directory?
+      ASSIGN(_directory, [_fm currentDirectoryPath]);
     }
+  else
+    {
+      ASSIGN(_directory, path);
+    }
+}
+
+- (void) _updateDefaultDirectory
+{
+  [[NSUserDefaults standardUserDefaults]
+      setObject: _directory
+      forKey: @"NSDefaultOpenDirectory"];
 }
 
 - (void) _resetDefaults
@@ -410,12 +411,19 @@ setPath(NSBrowser *browser, NSString *path)
   [self _setDefaultDirectory];
   [self setPrompt: @"Name:"];
   [self setTitle: @"Save"];
-  [self setRequiredFileType: @""];
+  [self setAllowedFileTypes: nil];
   [self setTreatsFilePackagesAsDirectories: NO];
   [self setDelegate: nil];
   [self setAccessoryView: nil];
 }
 
+- (void) _reloadBrowser
+{
+  NSString *path = [_browser path];
+  [_browser loadColumnZero];
+  setPath(_browser, path);
+}
+
 //
 // Methods invoked by button press
 //
@@ -555,7 +563,10 @@ selectCellWithString: (NSString*)title
 {
   if (path == nil)
     {
-      [self _setDefaultDirectory];
+      if (_directory == nil)
+        {
+	  [self _setDefaultDirectory];
+	}
     }
   else
     {
@@ -567,6 +578,8 @@ selectCellWithString: (NSString*)title
   setPath(_browser, _fullFileName);
 
   [self _selectCellName: filename];
+  [_form abortEditing]; // FIXME This is necessary because we cannot set a
+                        // cell's value while it is being edited.
   [[_form cellAtIndex: 0] setStringValue: filename];
   [_form selectTextAtIndex: 0];
   [_form setNeedsDisplay: YES];
@@ -577,24 +590,11 @@ selectCellWithString: (NSString*)title
 }
 
 - (BOOL) _shouldShowExtension: (NSString *)extension
-			isDir: (BOOL *)isDir;
 {
-  if (*isDir == NO)
-    {
-      if (_requiredFileType != nil && [_requiredFileType length] != 0
-          && [extension isEqualToString: _requiredFileType] == NO)
-	return NO;
-    }
-  else if ([extension length] == 0)
-    {
-      /* Automatic YES */
-    }
-  else if (_treatsFilePackagesAsDirectories == NO)
-    {
-      if (_requiredFileType == nil || [_requiredFileType length] == 0
-              || [extension isEqualToString: _requiredFileType] == YES)
-	  *isDir = NO;
-    }
+  if (_allowedFileTypes != nil
+      && [_allowedFileTypes indexOfObject: extension] == NSNotFound
+      && [_allowedFileTypes indexOfObject: @""] == NSNotFound)
+    return NO;
 
   return YES;
 }
@@ -680,7 +680,7 @@ selectCellWithString: (NSString*)title
   [[NSNotificationCenter defaultCenter] removeObserver: self];
   TEST_RELEASE (_fullFileName);
   TEST_RELEASE (_directory);  
-  TEST_RELEASE (_requiredFileType);
+  TEST_RELEASE (_allowedFileTypes);
 
   [super dealloc];
 }
@@ -696,7 +696,7 @@ selectCellWithString: (NSString*)title
  * All these are set automatically  
   _directory = nil;
   _fullFileName = nil;
-  _requiredFileType = nil;
+  _allowedFileTypes = nil;
   _delegate = nil;
   
   _treatsFilePackagesAsDirectories = NO;
@@ -921,24 +921,81 @@ selectCellWithString: (NSString*)title
    is used for another file type within the application.  If
    you do not invoke it, or set it to empty string or nil, no
    extension will be appended, indicated by an empty string
-   returned from -requiredFileType .</p><p>See Also: -requiredFileType</p>
+   returned from -requiredFileType.</p><p>This method is equivalent
+   to calling -setAllowedFileTypes: with an array containing only
+   fileType.</p><p>See Also: -requiredFileType</p>
  */
 - (void) setRequiredFileType: (NSString*)fileType
 {
-  ASSIGN(_requiredFileType, fileType);
+  NSArray *fileTypes;
+
+  if ([fileType length] == 0)
+    fileTypes = nil;
+  else
+    fileTypes = [NSArray arrayWithObject: fileType];
+  [self setAllowedFileTypes: fileTypes];
 }
 
-/**<p>Returns the required file type. The default, indicated by empty string,
- * is no required file type.</p><p>See Also: -setRequiredFileType:</p>
+/**<p>Returns the required file type.  The default, indicated by an empty
+ * string, is no required file type.</p><p>This method is equivalent to
+ * calling -allowedFileTypes and returning the first element of the list
+ * of allowed types, or the empty string if there are none.</p>
+ * <p>See Also: -setRequiredFileType:</p>
  */
 - (NSString*) requiredFileType
 {
-  return _requiredFileType;
+  if ([_allowedFileTypes count] > 0)
+    return [_allowedFileTypes objectAtIndex: 0];
+  else
+    return @"";
 }
 
+/**<p> Specifies the allowed types, i.e., file name extensions to
+   be appended to any selected files that don't already have one
+   of those extensions.  The elements of the array should be strings
+   that do not include the period that begins the extension.  Invoke
+   this method each time the Save panel is used for another file type
+   within the application.  If you do not invoke it, or set it to an
+   empty array or nil, no extension will be appended, indicated by nil
+   returned from -allowedFileTypes.</p><p>See Also: -allowedFileTypes</p>
+ */
 - (void) setAllowedFileTypes: (NSArray *)types
 {
-  // FIXME
+  if (types != _allowedFileTypes)
+    {
+      BOOL hasAllowedExtension = NO;
+      NSString *filename, *extension;
+
+      filename = [[_form cellAtIndex: 0] stringValue];
+      extension = [filename pathExtension];
+      if ([extension length] && [_allowedFileTypes count] &&
+	  [_allowedFileTypes indexOfObject: extension] != NSNotFound)
+	hasAllowedExtension = YES;
+
+      if ([types count] == 0)
+   	DESTROY(_allowedFileTypes);
+      else
+	ASSIGN(_allowedFileTypes, types);
+      [self _reloadBrowser];
+
+      if (hasAllowedExtension && [types count] &&
+	  [types indexOfObject: extension] == NSNotFound &&
+	  [types indexOfObject: @""] == NSNotFound)
+        {
+	  extension = [types objectAtIndex: 0];
+	  filename = [filename stringByDeletingPathExtension];
+	  filename = [filename stringByAppendingPathExtension: extension];
+	  [[_form cellAtIndex: 0] setStringValue: filename];
+	}
+    }
+}
+
+/**<p>Returns an array of the allowed file types. The default, indicated by
+ * nil, is any file type is allowed.</p><p>See Also: -setAllowedFileTypes:</p>
+ */
+- (NSArray *) allowedFileTypes
+{
+  return _allowedFileTypes;
 }
 
 - (void) setAllowsOtherFileTypes: (BOOL)flag
@@ -946,12 +1003,6 @@ selectCellWithString: (NSString*)title
   _allowsOtherFileTypes = flag;
 }
 
-- (NSArray *) allowedFileTypes
-{
-  // FIXME
-  return nil;
-}
-
 - (BOOL) allowsOtherFileTypes
 {
   return _allowsOtherFileTypes;
@@ -973,7 +1024,11 @@ selectCellWithString: (NSString*)title
  */
 - (void) setTreatsFilePackagesAsDirectories: (BOOL)flag
 {
-  _treatsFilePackagesAsDirectories = flag;
+  if (flag != _treatsFilePackagesAsDirectories)
+    {
+      _treatsFilePackagesAsDirectories = flag;
+      [self _reloadBrowser];
+    }
 }
 
 /**<p> Validates and possibly reloads the browser columns that are visible 
@@ -1069,17 +1124,27 @@ selectCellWithString: (NSString*)title
  */
 - (NSString*) filename
 {
+  NSString *fileType;
+
   if (_fullFileName == nil)
    return @"";
 
-  if (_requiredFileType == nil || [_requiredFileType isEqual: @""] == YES)
+  if (_allowedFileTypes == nil ||
+      [_allowedFileTypes indexOfObject: @""] != NSNotFound)
     return _fullFileName;
 
-  // add filetype extension only if the filename does not include it already
-  if ([[_fullFileName pathExtension] isEqual: _requiredFileType] == YES)
-    return _fullFileName;
+  /* add filetype extension only if the filename does not include an
+     allowed one already */
+  fileType = [_fullFileName pathExtension];
+  if ([_allowedFileTypes indexOfObject: fileType] != NSNotFound)
+    {
+      return _fullFileName;
+    }
   else
-    return [_fullFileName stringByAppendingPathExtension: _requiredFileType];
+    {
+      fileType = [_allowedFileTypes objectAtIndex: 0];
+      return [_fullFileName stringByAppendingPathExtension: fileType];
+    }
 }
 
 - (NSURL *) URL
@@ -1094,6 +1159,7 @@ selectCellWithString: (NSString*)title
 - (void) cancel: (id)sender
 {
   ASSIGN(_directory, pathToColumn(_browser, [_browser lastColumn]));
+  [self _updateDefaultDirectory];
   [NSApp stopModalWithCode: NSCancelButton];
   [_okButton setEnabled: NO];
   [self close];
@@ -1124,15 +1190,11 @@ selectCellWithString: (NSString*)title
 
   ASSIGN (_directory, pathToColumn(_browser, [_browser lastColumn]));
   filename = [[_form cellAtIndex: 0] stringValue];
-  if ([filename isAbsolutePath] == YES)
+  if ([filename isAbsolutePath] == NO)
     {
-      ASSIGN (_fullFileName, filename);
-    }
-  else
-    {
-      ASSIGN (_fullFileName, [_directory stringByAppendingPathComponent:
-				       filename]);
+      filename = [_directory stringByAppendingPathComponent: filename];
     }
+  ASSIGN (_fullFileName, [filename stringByStandardizingPath]);
 
   if (_delegateHasUserEnteredFilename)
     {
@@ -1207,6 +1269,7 @@ selectCellWithString: (NSString*)title
     if (![_delegate panel: self isValidFilename: [self filename]])
       return;
 
+  [self _updateDefaultDirectory];
   [NSApp stopModalWithCode: NSOKButton];
   [_okButton setEnabled: NO];
   [self close];
@@ -1402,6 +1465,7 @@ selectCellWithString: (NSString*)title
 // NSSavePanel browser delegate methods
 //
 @interface NSSavePanel (GSBrowserDelegate)
+- (void) browserDidScroll: (NSBrowser *)sender;
 - (void) browser: (NSBrowser*)sender
 createRowsForColumn: (int)column
         inMatrix: (NSMatrix*)matrix;
@@ -1434,6 +1498,11 @@ static int compareFilenames (id elem1, id elem2, void *context)
 
 
 @implementation NSSavePanel (GSBrowserDelegate)
+- (void) browserDidScroll: (NSBrowser *)sender
+{
+  [self validateVisibleColumns];
+}
+
 - (void) browser: (NSBrowser*)sender
 createRowsForColumn: (int)column
 	inMatrix: (NSMatrix*)matrix
@@ -1448,11 +1517,12 @@ createRowsForColumn: (int)column
   unsigned               base_frac = 1;
   BOOL                   display_progress = NO;
   NSString              *progressString = nil;
+  NSWorkspace		*ws;
   /* We create lot of objects in this method, so we use a pool */
   NSAutoreleasePool     *pool;
 
   pool = [NSAutoreleasePool new];
-  
+  ws = [NSWorkspace sharedWorkspace];
   path = pathToColumn(_browser, column);
 #if	defined(__MINGW32__)
   if (column == 0)
@@ -1460,7 +1530,7 @@ createRowsForColumn: (int)column
       NSMutableArray	*m;
       unsigned		i;
 
-      files = [[NSWorkspace sharedWorkspace] mountedLocalVolumePaths];
+      files = [ws mountedLocalVolumePaths];
       m = [files mutableCopy];
       i = [m count];
       while (i-- > 0)
@@ -1561,7 +1631,7 @@ createRowsForColumn: (int)column
       progressString = [@"Reading Directory " stringByAppendingString: path];
       [super setTitle: progressString];
       // Is the following really safe? 
-      [GSCurrentContext() flushGraphics];
+      [self flushWindow];
     }
 
   //TODO: Sort after creation of matrix so we do not sort 
@@ -1580,12 +1650,12 @@ createRowsForColumn: (int)column
     {
       // Update displayed message if needed
       if (display_progress && (i > (base_frac * (reached_frac + 1))))
-	{
-	  reached_frac++;
-	  progressString = [progressString stringByAppendingString: @"."];
-	  [super setTitle: progressString];
-	  [GSCurrentContext() flushGraphics];
-	}
+        {
+          reached_frac++;
+          progressString = [progressString stringByAppendingString: @"."];
+          [super setTitle: progressString];
+          [self flushWindow];
+        }
       // Now the real code
       file = [files objectAtIndex: i];
       extension = [file pathExtension];
@@ -1593,16 +1663,19 @@ createRowsForColumn: (int)column
       pathAndFile = [path stringByAppendingPathComponent: file];
       exists = [_fm fileExistsAtPath: pathAndFile 
 		    isDirectory: &isDir];
-      
+      if (isDir && !_treatsFilePackagesAsDirectories
+	&& [ws isFilePackageAtPath: pathAndFile])
+        {
+	  isDir = NO;
+	}
       if (_delegateHasShowFilenameFilter)
 	{
-	  exists = [_delegate panel: self
-			      shouldShowFilename: pathAndFile];
+	  exists = [_delegate panel: self shouldShowFilename: pathAndFile];
 	}
 
-      if (exists)
+      if (exists && !isDir)
 	{
-	  exists = [self _shouldShowExtension: extension isDir: &isDir];
+	  exists = [self _shouldShowExtension: extension];
 	}
       
       if (exists)
@@ -1635,7 +1708,7 @@ createRowsForColumn: (int)column
   if (display_progress)
     {
       [super setTitle: @""];
-      [GSCurrentContext() flushGraphics];
+      [self flushWindow];
     }
 
   RELEASE (pool);
@@ -1644,6 +1717,10 @@ createRowsForColumn: (int)column
 - (BOOL) browser: (NSBrowser*)sender
    isColumnValid: (int)column
 {
+  /*
+   * FIXME This code doesn't handle the case where the delegate now wants
+   *       to show additional files, which were not displayed before.
+   */
   NSArray	*cells = [[sender matrixInColumn: column] cells];
   unsigned	count = [cells count], i;
   NSString	*path = pathToColumn(sender, column);
