@@ -49,6 +49,10 @@
 #include "AppKit/PSOperators.h"
 #include "GNUstepGUI/GSTheme.h"
 
+@interface NSClipView (Private)
+- (void) _scrollToPoint: (NSPoint)aPoint;
+@end
+
 //
 // For nib compatibility, this is used to properly
 // initialize the object from a OS X nib file in initWithCoder:.
@@ -232,8 +236,10 @@ static float scrollerWidth;
       NSView *docView = [aView documentView];
 
       [_contentView removeFromSuperview];
+      [self addSubview: aView];
+      // This must be done after adding it as a subview,
+      // otherwise it will get unset again.
       _contentView = aView;
-      [self addSubview: _contentView];
 
       if (docView != nil)
         {
@@ -250,6 +256,14 @@ static float scrollerWidth;
     {
       _contentView = nil;
     }
+  if (aView == _headerClipView)
+    {
+      _headerClipView = nil;
+    }
+  if (aView == _cornerView)
+    {
+      _cornerView = nil;
+    }
   [super removeSubview: aView];
 }
 
@@ -406,7 +420,7 @@ static float scrollerWidth;
 
   /* scrollToPoint: will call reflectScrolledClipView:, which will
    * update rules, headers, and scrollers.  */
-  [_contentView scrollToPoint: point];
+  [_contentView _scrollToPoint: point];
 }
 
 - (void) keyDown: (NSEvent *)theEvent
@@ -462,7 +476,7 @@ static float scrollerWidth;
       amount = -amount;
     }
   point.y = clipViewBounds.origin.y - amount;
-  [_contentView scrollToPoint: point];
+  [_contentView _scrollToPoint: point];
 }
 
 
@@ -490,7 +504,7 @@ static float scrollerWidth;
       amount = -amount;
     }
   point.y = clipViewBounds.origin.y + amount;
-  [_contentView scrollToPoint: point];
+  [_contentView _scrollToPoint: point];
 }
 
 /**
@@ -534,7 +548,7 @@ static float scrollerWidth;
       amount = -amount;
     }
   point.y = clipViewBounds.origin.y - amount;
-  [_contentView scrollToPoint: point];
+  [_contentView _scrollToPoint: point];
 }
 
 /**
@@ -577,7 +591,7 @@ static float scrollerWidth;
       amount = -amount;
     }
   point.y = clipViewBounds.origin.y + amount;
-  [_contentView scrollToPoint: point];
+  [_contentView _scrollToPoint: point];
 }
 
 - (void) _doScroll: (NSScroller*)scroller
@@ -686,7 +700,7 @@ static float scrollerWidth;
 
   /* scrollToPoint will call reflectScrollerClipView, and that will
    * update scrollers, rulers and headers */
-  [_contentView scrollToPoint: point];
+  [_contentView _scrollToPoint: point];
 }
 
 //
@@ -1507,12 +1521,7 @@ static float scrollerWidth;
               [content setFrame: frame];
             }
 
-          // FIXME: No idea what is going on here.
-          // retain the view and reset the content view...
-          RETAIN(content);
           [self setContentView: content];
-          RELEASE(content);
-          ASSIGN(_contentView, content);
         }
      
       if (hScroller != nil && _hasHorizScroller)
@@ -1529,8 +1538,7 @@ static float scrollerWidth;
         {
           _hasHeaderView = YES;
           _hasCornerView = YES;          
-          ASSIGN(_headerClipView,
-            [aDecoder decodeObjectForKey: @"NSHeaderClipView"]);
+          _headerClipView = [aDecoder decodeObjectForKey: @"NSHeaderClipView"];
         }
 
       [self tile];
@@ -1539,7 +1547,7 @@ static float scrollerWidth;
     {
       int version = [aDecoder versionForClassName: @"NSScrollView"];
       NSDebugLLog(@"NSScrollView", @"NSScrollView: start decoding\n");
-      [aDecoder decodeValueOfObjCType: @encode(id) at: &_contentView];
+      _contentView = [aDecoder decodeObject];
       [aDecoder decodeValueOfObjCType: @encode(NSBorderType) at: &_borderType];
       [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &_scrollsDynamically];
       [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &_rulersVisible];
@@ -1568,7 +1576,7 @@ static float scrollerWidth;
         {
           [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &_hasHeaderView];
           if (_hasHeaderView)
-            [aDecoder decodeValueOfObjCType: @encode(id) at: &_headerClipView];
+            _headerClipView = [aDecoder decodeObject];
       
           [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &_hasCornerView];
         }
