@@ -51,6 +51,7 @@
 #include <Foundation/NSPathUtilities.h>
 #include <Foundation/NSUserDefaults.h>
 #include <Foundation/NSTask.h>
+#include <GNUstepBase/NSTask+GS.h>
 #include <Foundation/NSException.h>
 #include <Foundation/NSFileManager.h>
 #include <Foundation/NSNotificationQueue.h>
@@ -1037,6 +1038,7 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
     }
   else
     {
+      *appName = nil;
       return NO;
     }
 }
@@ -1045,10 +1047,14 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
 {
   NSImage	*image = nil;
   NSString	*pathExtension = [[fullPath pathExtension] lowercaseString];
+  NSFileManager	*mgr = [NSFileManager defaultManager];
+  NSDictionary	*attributes;
+  NSString	*fileType;
 
-  if ([self isFilePackageAtPath: fullPath])
+  attributes = [mgr fileAttributesAtPath: fullPath traverseLink: YES];
+  fileType = [attributes objectForKey: NSFileType];
+  if ([fileType isEqual: NSFileTypeDirectory] == YES)
     {
-      NSFileManager	*mgr = [NSFileManager defaultManager];
       NSString		*iconPath = nil;
       BOOL		isApplication = NO;
       
@@ -1203,13 +1209,29 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
 {
   NSFileManager	*mgr = [NSFileManager defaultManager];
   NSDictionary	*attributes;
-  NSString	*fileType;
+  NSString	*fileType, *extension;
 
   attributes = [mgr fileAttributesAtPath: fullPath traverseLink: YES];
   fileType = [attributes objectForKey: NSFileType];
   if ([fileType isEqual: NSFileTypeDirectory] == YES)
     {
-      return YES;
+      /*
+       * We return YES here exactly when getInfoForFile:application:type:
+       * considers the directory an application or a plain file
+       */
+      extension = [fullPath pathExtension];
+      if ([extension isEqualToString: @"app"]
+	|| [extension isEqualToString: @"debug"]
+	|| [extension isEqualToString: @"profile"]
+        || [extension isEqualToString: @"bundle"])
+        {
+	  return YES;
+	}
+      else if ([extension length] > 0
+        && [self getBestAppInRole: nil forExtension: extension] != nil)
+        {
+	  return YES;
+        }
     }
   return NO;
 }
@@ -1252,14 +1274,7 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
    */
   if (path == nil)
     {
-#ifdef GNUSTEP_BASE_LIBRARY
-      path = RETAIN([[NSSearchPathForDirectoriesInDomains(
-	  GSToolsDirectory, NSSystemDomainMask, YES) objectAtIndex: 0] 
-		 stringByAppendingPathComponent: @"make_services"]);
-#else
-      path = RETAIN([@GNUSTEP_TOOLS_NO_DESTDIR
-		      stringByAppendingPathComponent: @"make_services"]);
-#endif
+      path = [NSTask launchPathForTool: @"make_services"];
     }
   task = [NSTask launchedTaskWithLaunchPath: path
 				  arguments: nil];
@@ -1609,6 +1624,7 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
   FILE		*fptr = fopen("/etc/mtab", "r");
   struct mntent	*m;
 
+
   names = [NSMutableArray arrayWithCapacity: 8];
   while ((m = getmntent(fptr)) != 0)
     {
@@ -1619,12 +1635,35 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
       [names addObject: path];
     }
 #else
-
-  // FIXME This is system specific
-  NSString	*mtab = [NSString stringWithContentsOfFile: @"/etc/mtab"];
-  NSArray	*mounts = [mtab componentsSeparatedByString: @"\n"];
+  NSString	*mtabPath;
+  NSString	*mtab;
+  NSArray	*mounts, *reservedMountNames;
   unsigned int	i;
 
+  // get mount table...
+  mtabPath = [[NSUserDefaults standardUserDefaults] objectForKey:@"GSMtabPath"];
+  if (mtabPath == nil)
+    {
+      mtabPath = @"/etc/mtab";
+    }
+  
+  // get reserved names....
+  reservedMountNames = [[NSUserDefaults standardUserDefaults] objectForKey: @"GSReservedMountNames"];
+  if(reservedMountNames == nil)
+    {
+      reservedMountNames = [NSArray arrayWithObjects: 
+				      @"proc",@"devpts",
+				    @"shm",@"usbdevfs",
+				    @"devpts",@"sysfs",
+				    @"tmpfs",@"procbususb",
+				    @"udev",nil];
+      [[NSUserDefaults standardUserDefaults] setObject: reservedMountNames 
+					     forKey: @"GSReservedMountNames"];
+    }
+
+  mtab = [NSString stringWithContentsOfFile:mtabPath];
+  mounts = [mtab componentsSeparatedByString: @"\n"];
+
   names = [NSMutableArray arrayWithCapacity: [mounts count]];
   for (i = 0; i < [mounts count]; i++)
     {
@@ -1638,9 +1677,7 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
             {          
               NSString	*type = [parts objectAtIndex: 2];
               
-              if ([type isEqualToString: @"proc"] == NO
-	          && [type isEqualToString: @"devpts"] == NO
-	          && [type isEqualToString: @"shm"] == NO)
+              if ([reservedMountNames containsObject: type] == NO)
               {
 	         [names addObject: [parts objectAtIndex: 1]];
 	      }
@@ -1849,7 +1886,14 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
     }
   
   iconPath = [[bundle infoDictionary] objectForKey: @"NSIcon"];
-  
+  if(iconPath == nil)
+    {
+      /*
+       * Try the CFBundleIconFile property.
+       */
+      iconPath = [[bundle infoDictionary] objectForKey: @"CFBundleIconFile"];
+    }
+
   if (iconPath && [iconPath isAbsolutePath] == NO)
     {
       NSString *file = iconPath;
@@ -1875,7 +1919,7 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
    * try 'wrapper/app.png'
    */
   if (iconPath == nil)
-    {
+    {      
       NSString *str;
 
       str = [fullPath lastPathComponent];
@@ -1887,11 +1931,15 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
 	  iconPath = [iconPath stringByAppendingPathExtension: @"tiff"];
 	  if ([mgr isReadableFileAtPath: iconPath] == NO)
 	    {
-	      iconPath = nil;
+	      iconPath = [iconPath stringByAppendingPathExtension: @"icns"];
+	      if ([mgr isReadableFileAtPath: iconPath] == NO)
+		{		  
+		  iconPath = nil;
+		}
 	    }
         }
     }
-  
+
   if (iconPath != nil)
     {
       image = [self _saveImageFor: iconPath];
@@ -2042,6 +2090,11 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
   NSDictionary	*typeInfo = [extInfo objectForKey: appName];
   NSString	*file = [typeInfo objectForKey: @"NSIcon"];
 
+  if(file == nil)
+    {
+      file = [typeInfo objectForKey: @"CFBundleTypeIconFile"];
+    }
+
   if (file && [file length] != 0)
     {
       if ([file isAbsolutePath] == NO)
@@ -2324,7 +2377,7 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
 
       /*
        * If the requested role is 'nil', we can accept an app that is either
-       * an Editor (preferred) or a Viewer.
+       * an Editor (preferred) or a Viewer, or unknown.
        */
       while ((appName = [enumerator nextObject]) != nil)
 	{
@@ -2332,7 +2385,10 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
 
 	  info = [apps objectForKey: appName];
 	  str = [info objectForKey: @"NSRole"];
-	  if (str == nil || [str isEqualToString: @"Editor"])
+	  /* NB. If str is nil or an empty string, there is no role set,
+	   * and we treat this as an Editor since the role is unrestricted.
+	   */
+	  if ([str length] == 0 || [str isEqualToString: @"Editor"])
 	    {
 	      if (app != 0)
 		{
@@ -2340,7 +2396,7 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
 		}
 	      return YES;
 	    }
-	  else if ([str isEqualToString: @"Viewer"])
+	  if ([str isEqualToString: @"Viewer"])
 	    {
 	      if (app != 0)
 		{
