@@ -13,16 +13,16 @@
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 3
    of the License, or (at your option) any later version.
-    
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public  
+   You should have received a copy of the GNU General Public
    License along with this library; see the file COPYING.
-   If not, see <http://www.gnu.org/licenses/> or write to the 
-   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   If not, see <http://www.gnu.org/licenses/> or write to the
+   Free Software Foundation, 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
 */
 
@@ -177,12 +177,12 @@ main(int argc, char** argv, char **env_c)
 		    {
 		      validateEntry(svcs, file);
 		    }
-		  else if (verbose >= 0)
+		  else if (verbose > 0)
 		    {
 		      NSLog(@"bad info - %@", file);
 		    }
 		}
-	      else if (verbose >= 0)
+	      else if (verbose > 0)
 		{
 		  NSLog(@"bad info - %@", file);
 		}
@@ -201,7 +201,7 @@ main(int argc, char** argv, char **env_c)
   usrRoot = [usrRoot stringByAppendingPathComponent: @"Services"];
   if (CheckDirectory(usrRoot) == NO)
     {
-      if (verbose >= 0)
+      if (verbose > 0)
 	NSLog(@"couldn't create %@", usrRoot);
       [pool release];
       exit(EXIT_FAILURE);
@@ -221,7 +221,10 @@ main(int argc, char** argv, char **env_c)
     NSAllApplicationsDirectory, NSAllDomainsMask, YES) objectEnumerator];
   while ((path = [enumerator nextObject]) != nil)
     {
-      scanApplications(services, path);
+      if ([path hasPrefix: @"."] == NO)
+ 	{
+          scanApplications(services, path);
+	}
     }
 
   /*
@@ -258,7 +261,7 @@ main(int argc, char** argv, char **env_c)
       data = [NSSerializer serializePropertyList: fullMap];
       if ([data writeToFile: str atomically: YES] == NO)
 	{
-	  if (verbose >= 0)
+	  if (verbose > 0)
 	    NSLog(@"couldn't write %@", str);
 	  [pool release];
 	  exit(EXIT_FAILURE);
@@ -282,7 +285,7 @@ main(int argc, char** argv, char **env_c)
       data = [NSSerializer serializePropertyList: applicationMap];
       if ([data writeToFile: str atomically: YES] == NO)
 	{
-	  if (verbose >= 0)
+	  if (verbose > 0)
 	    NSLog(@"couldn't write %@", str);
 	  [pool release];
 	  exit(EXIT_FAILURE);
@@ -308,11 +311,16 @@ static void addExtensionsForApplication(NSDictionary *info, NSString *app)
 
 
   o0 = [info objectForKey: @"NSTypes"];
+  if (o0 == nil)
+    {
+      o0 = [info objectForKey: @"CFBundleDocumentTypes"];
+    }
+
   if (o0)
     {
       if ([o0 isKindOfClass: aClass] == NO)
         {
-	  if (verbose >= 0)
+	  if (verbose > 0)
 	    NSLog(@"bad app NSTypes (not an array) - %@", app);
           return;
         }
@@ -327,7 +335,7 @@ static void addExtensionsForApplication(NSDictionary *info, NSString *app)
 
           if ([o1 isKindOfClass: dClass] == NO)
             {
-	      if (verbose >= 0)
+	      if (verbose > 0)
 		NSLog(@"bad app NSTypes (type not a dictionary) - %@", app);
               return;
             }
@@ -336,13 +344,18 @@ static void addExtensionsForApplication(NSDictionary *info, NSString *app)
 	   */
           t = (NSDictionary*)o1;
           o1 = [t objectForKey: @"NSUnixExtensions"];
+
+	  if(o1 == nil)
+	    {
+	      o1 = [t objectForKey: @"CFBundleTypeExtensions"];
+	    }
           if (o1 == nil)
             {
               continue;
             }
           if ([o1 isKindOfClass: aClass] == NO)
             {
-	      if (verbose >= 0)
+	      if (verbose > 0)
 		NSLog(@"bad app NSType (extensions not an array) - %@", app);
               return;
             }
@@ -356,7 +369,7 @@ static void addExtensionsForApplication(NSDictionary *info, NSString *app)
               e = [[a1 objectAtIndex: j] lowercaseString];
 	      if ([e length] == 0)
 		{
-		  if (verbose >= 0)
+		  if (verbose > 0)
 		    NSLog(@"Illegal (nul) extension ignored for - %@", app);
 		  return;
 		}
@@ -378,13 +391,17 @@ static void addExtensionsForApplication(NSDictionary *info, NSString *app)
       NSDictionary	*extensions;
 
       o0 = [info objectForKey: @"NSExtensions"];
+      if(o0 == nil)
+	{
+	  o0 = [info objectForKey: @"CFBundleTypeExtensions"];
+	}
       if (o0 == nil)
         {
           return;
         }
       if ([o0 isKindOfClass: dClass] == NO)
         {
-	  if (verbose >= 0)
+	  if (verbose > 0)
 	    NSLog(@"bad app NSExtensions (not a dictionary) - %@", app);
           return;
         }
@@ -399,7 +416,7 @@ static void addExtensionsForApplication(NSDictionary *info, NSString *app)
 
           if ([tmp isKindOfClass: dClass] == NO)
 	    {
-	      if (verbose >= 0)
+	      if (verbose > 0)
 		NSLog(@"bad app NSExtensions (value isn't a dictionary) - %@",
 		      app);
               continue;
@@ -416,6 +433,7 @@ static void addExtensionsForApplication(NSDictionary *info, NSString *app)
     }
 }
 
+#if	0	// UNUSED
 static void
 scanDirectory(NSMutableDictionary *services, NSString *path)
 {
@@ -431,6 +449,13 @@ scanDirectory(NSMutableDictionary *services, NSString *path)
       NSString	*newPath;
       BOOL	isDir;
 
+      /*
+       *	Ignore anything with a leading dot.
+       */
+      if ([name hasPrefix: @"."] == YES)
+ 	{
+	  continue;
+	}
       if (ext != nil
 	&& ([ext isEqualToString: @"app"] || [ext isEqualToString: @"debug"]
 	|| [ext isEqualToString: @"profile"]))
@@ -458,7 +483,7 @@ scanDirectory(NSMutableDictionary *services, NSString *path)
                    * this name, we skip this one - the first one takes
                    * precedence.
                    */
-		  if (verbose >= 0)
+		  if (verbose > 0)
 		    NSLog(@"duplicate app (%@) at '%@' and '%@'",
 			  name, oldPath, newPath);
                   continue;
@@ -487,12 +512,12 @@ scanDirectory(NSMutableDictionary *services, NSString *path)
 
 		  addExtensionsForApplication(info, name);
 		}
-	      else if (verbose >= 0)
+	      else if (verbose > 0)
 		{
 		  NSLog(@"bad app info - %@", newPath);
 		}
 	    }
-	  else if (verbose >= 0)
+	  else if (verbose > 0)
 	    {
 	      NSLog(@"bad application - %@", newPath);
 	    }
@@ -522,17 +547,17 @@ scanDirectory(NSMutableDictionary *services, NSString *path)
 			  [services setObject: entry forKey: newPath];
 			}
 		    }
-		  else if (verbose >= 0)
+		  else if (verbose > 0)
 		    {
 		      NSLog(@"missing info - %@", newPath);
 		    }
 		}
-	      else if (verbose >= 0)
+	      else if (verbose > 0)
 		{
 		  NSLog(@"bad service info - %@", newPath);
 		}
 	    }
-	  else if (verbose >= 0)
+	  else if (verbose > 0)
 	    {
 	      NSLog(@"bad services bundle - %@", newPath);
 	    }
@@ -549,6 +574,7 @@ scanDirectory(NSMutableDictionary *services, NSString *path)
     }
   [arp release];
 }
+#endif
 
 static void
 scanApplications(NSMutableDictionary *services, NSString *path)
@@ -565,6 +591,13 @@ scanApplications(NSMutableDictionary *services, NSString *path)
       NSString	*newPath;
       BOOL	isDir;
 
+      /*
+       *	Ignore anything with a leading dot.
+       */
+      if ([name hasPrefix: @"."] == YES)
+ 	{
+	  continue;
+	}
       if (ext != nil
 	&& ([ext isEqualToString: @"app"] || [ext isEqualToString: @"debug"]
 	|| [ext isEqualToString: @"profile"]))
@@ -592,7 +625,7 @@ scanApplications(NSMutableDictionary *services, NSString *path)
                    * this name, we skip this one - the first one takes
                    * precedence.
                    */
-		  if (verbose >= 0)
+		  if (verbose > 0)
 		    NSLog(@"duplicate app (%@) at '%@' and '%@'",
 			  name, oldPath, newPath);
                   continue;
@@ -621,12 +654,12 @@ scanApplications(NSMutableDictionary *services, NSString *path)
 
 		  addExtensionsForApplication(info, name);
 		}
-	      else if (verbose >= 0)
+	      else if (verbose > 0)
 		{
 		  NSLog(@"bad app info - %@", newPath);
 		}
 	    }
-	  else if (verbose >= 0)
+	  else if (verbose > 0)
 	    {
 	      NSLog(@"bad application - %@", newPath);
 	    }
@@ -688,7 +721,7 @@ scanDynamic(NSMutableDictionary *services, NSString *path)
 		}
 	    }
 	}
-      else if (verbose >= 0)
+      else if (verbose > 0)
 	{
 	  NSLog(@"bad app info - %@", infPath);
 	}
@@ -711,6 +744,13 @@ scanServices(NSMutableDictionary *services, NSString *path)
       NSString	*newPath;
       BOOL	isDir;
 
+      /*
+       *	Ignore anything with a leading dot.
+       */
+      if ([name hasPrefix: @"."] == YES)
+ 	{
+	  continue;
+	}
       if (ext != nil && [ext isEqualToString: @"service"])
 	{
 	  newPath = [path stringByAppendingPathComponent: name];
@@ -736,17 +776,17 @@ scanServices(NSMutableDictionary *services, NSString *path)
 			  [services setObject: entry forKey: newPath];
 			}
 		    }
-		  else if (verbose >= 0)
+		  else if (verbose > 0)
 		    {
 		      NSLog(@"missing info - %@", newPath);
 		    }
 		}
-	      else if (verbose >= 0)
+	      else if (verbose > 0)
 		{
 		  NSLog(@"bad service info - %@", newPath);
 		}
 	    }
-	  else if (verbose >= 0)
+	  else if (verbose > 0)
 	    {
 	      NSLog(@"bad services bundle - %@", newPath);
 	    }
@@ -773,7 +813,7 @@ validateEntry(id svcs, NSString *path)
 
   if ([svcs isKindOfClass: aClass] == NO)
     {
-      if (verbose >= 0)
+      if (verbose > 0)
 	NSLog(@"NSServices entry not an array - %@", path);
       return nil;
     }
@@ -796,7 +836,7 @@ validateEntry(id svcs, NSString *path)
 	      [newServices addObject: newService];
 	    }
 	}
-      else if (verbose >= 0)
+      else if (verbose > 0)
 	{
 	  NSLog(@"NSServices entry %u not a dictionary - %@",
 	    pos, path);
@@ -833,7 +873,7 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 	@"string", @"NSDeviceDependent",
 	@"array", @"NSLanguages",
 	@"string", @"NSSpellChecker",
-	nil]; 
+	nil];
       [fields retain];
     }
 
@@ -850,7 +890,7 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 
       if (type == nil)
 	{
-	  if (verbose >= 0)
+	  if (verbose > 0)
 	    NSLog(@"NSServices entry %u spurious field (%@)- %@", pos, k, path);
 	}
       else
@@ -860,7 +900,7 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 	    {
 	      if ([obj isKindOfClass: sClass] == NO)
 		{
-		  if (verbose >= 0)
+		  if (verbose > 0)
 		    NSLog(@"NSServices entry %u field %@ is not a string "
 			  @"- %@", pos, k, path);
 		  return nil;
@@ -873,7 +913,7 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 
 	      if ([obj isKindOfClass: aClass] == NO)
 		{
-		  if (verbose >= 0)
+		  if (verbose > 0)
 		    NSLog(@"NSServices entry %u field %@ is not an array "
 		    @"- %@", pos, k, path);
 		  return nil;
@@ -881,7 +921,7 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 	      a = (NSArray*)obj;
 	      if ([a count] == 0)
 		{
-		  if (verbose >= 0)
+		  if (verbose > 0)
 		    NSLog(@"NSServices entry %u field %@ is an empty array "
 			  @"- %@", pos, k, path);
 		}
@@ -893,7 +933,7 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 		    {
 		      if ([[a objectAtIndex: i] isKindOfClass: sClass] == NO)
 			{
-			  if (verbose >= 0)
+			  if (verbose > 0)
 			    NSLog(@"NSServices entry %u field %@ element %u is "
 				  @"not a string - %@", pos, k, i, path);
 			  return nil;
@@ -908,7 +948,7 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 
 	      if ([obj isKindOfClass: dClass] == NO)
 		{
-		  if (verbose >= 0)
+		  if (verbose > 0)
 		    NSLog(@"NSServices entry %u field %@ is not a dictionary "
 			  @"- %@", pos, k, path);
 		  return nil;
@@ -916,7 +956,7 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 	      d = (NSDictionary*)obj;
 	      if ([d objectForKey: @"default"] == nil)
 		{
-		  if (verbose >= 0)
+		  if (verbose > 0)
 		    NSLog(@"NSServices entry %u field %@ has no default value "
 			  @"- %@", pos, k, path);
 		}
@@ -928,7 +968,7 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 		    {
 		      if ([obj isKindOfClass: sClass] == NO)
 			{
-			  if (verbose >= 0)
+			  if (verbose > 0)
 			    NSLog(@"NSServices entry %u field %@ contains "
 				  @"non-string value - %@", pos, k, path);
 			  return nil;
@@ -962,14 +1002,14 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 	    && [str isEqualToString: @"NSMapFile"] == NO
 	    && [str isEqualToString: @"NSIdentity"] == NO)
 	  {
-	    if (verbose >= 0)
+	    if (verbose > 0)
 	      NSLog(@"NSServices entry %u bad input mechanism - %@", pos, path);
 	    return nil;
 	  }
 	}
       else if ([result objectForKey: @"NSPortName"] == nil)
 	{
-	  if (verbose >= 0)
+	  if (verbose > 0)
 	    NSLog(@"NSServices entry %u NSPortName missing - %@", pos, path);
 	  return nil;
 	}
@@ -978,8 +1018,9 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
       ret = [result objectForKey: @"NSReturnTypes"];
       if ([snd count] == 0 || [ret count] == 0)
 	{
-	  if (verbose >= 0)
-	    NSLog(@"NSServices entry %u types empty or missing - %@", pos, path);
+	  if (verbose > 0)
+	    NSLog(@"NSServices entry %u types empty or missing - %@",
+	      pos, path);
 	  return nil;
 	}
       else
@@ -1009,7 +1050,7 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 	}
       if (notPresent == NO)
 	{
-	  if (verbose)
+	  if (verbose > 0)
 	    {
 	      NSLog(@"Ignoring duplicate %u in %@ -\n%@", pos, path, result);
 	    }
@@ -1025,20 +1066,20 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 
       if ([result objectForKey: @"NSPortName"] == nil)
 	{
-	  if (verbose >= 0)
+	  if (verbose > 0)
 	    NSLog(@"NSServices entry %u NSPortName missing - %@", pos, path);
 	  return nil;
 	}
       if ([result objectForKey: @"NSSendTypes"] == nil
 	&& [result objectForKey: @"NSReturnTypes"] == nil)
 	{
-	  if (verbose >= 0)
+	  if (verbose > 0)
 	    NSLog(@"NSServices entry %u types missing - %@", pos, path);
 	  return nil;
 	}
       if ((item = [result objectForKey: @"NSMenuItem"]) == nil)
 	{
-	  if (verbose >= 0)
+	  if (verbose > 0)
 	    NSLog(@"NSServices entry %u NSMenuItem missing - %@", pos, path);
 	  return nil;
 	}
@@ -1067,7 +1108,7 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 	}
       if (used == NO)
 	{
-	  if (verbose)
+	  if (verbose > 0)
 	    {
 	      NSLog(@"Ignoring entry %u in %@ -\n%@", pos, path, result);
 	    }
@@ -1083,7 +1124,7 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 
       if ((item = [result objectForKey: @"NSMenuItem"]) == nil)
 	{
-	  if (verbose >= 0)
+	  if (verbose > 0)
 	    NSLog(@"NSServices entry %u NSMenuItem missing - %@", pos, path);
 	  return nil;
 	}
@@ -1111,7 +1152,7 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 	}
       if (used == NO)
 	{
-	  if (verbose)
+	  if (verbose > 0)
 	    {
 	      NSLog(@"Ignoring entry %u in %@ -\n%@", pos, path, result);
 	    }
@@ -1126,8 +1167,8 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 
       if ((item = [result objectForKey: @"NSLanguages"]) == nil)
 	{
-	  if (verbose >= 0)
-	    NSLog(@"NSServices entry %u NSLanguages missing - %@", pos, path);
+	  if (verbose > 0)
+	    NSLog(@"NSServices entry NSLanguages missing - %@", path);
 	  return nil;
 	}
       /*
@@ -1147,7 +1188,7 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 	}
       if (used == NO)
 	{
-	  if (verbose)
+	  if (verbose > 0)
 	    {
 	      NSLog(@"Ignoring entry %u in %@ -\n%@", pos, path, result);
 	    }
@@ -1156,11 +1197,11 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
     }
   else
     {
-      if (verbose >= 0)
+      if (verbose > 0)
 	NSLog(@"NSServices entry %u unknown service/filter - %@", pos, path);
       return nil;
     }
-  
+
   return result;
 }
 
